from texas_hold_em_utils.card import Card
from texas_hold_em_utils.sklansky import sklansky_rank


def test_sklansky_9_q_7_suited():
    rank_1 = "Q"
    suit_1 = "Spades"
    rank_2 = "7"
    suit_2 = "Spades"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_10_6_suited():
    rank_1 = "10"
    suit_1 = "Spades"
    rank_2 = "6"
    suit_2 = "Spades"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_9_5_suited():
    rank_1 = "9"
    suit_1 = "Spades"
    rank_2 = "5"
    suit_2 = "Spades"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_8_4_suited():
    rank_1 = "8"
    suit_1 = "Spades"
    rank_2 = "4"
    suit_2 = "Spades"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_6_3_suited():
    rank_1 = "3"
    suit_1 = "Spades"
    rank_2 = "6"
    suit_2 = "Spades"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_5_2_suited():
    rank_1 = "5"
    suit_1 = "Spades"
    rank_2 = "2"
    suit_2 = "Spades"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_q_8_off():
    rank_1 = "Q"
    suit_1 = "Spades"
    rank_2 = "8"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_6_4_off():
    rank_1 = "6"
    suit_1 = "Spades"
    rank_2 = "4"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_3_4_off():
    rank_1 = "3"
    suit_1 = "Spades"
    rank_2 = "4"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_9_3_2_off():
    rank_1 = "3"
    suit_1 = "Spades"
    rank_2 = "2"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 9


def test_sklansky_8_9_a_off():
    rank_1 = "9"
    suit_1 = "Diamonds"
    rank_2 = "A"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_9_k_off():
    rank_1 = "9"
    suit_1 = "Diamonds"
    rank_2 = "K"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_9_q_off():
    rank_1 = "9"
    suit_1 = "Diamonds"
    rank_2 = "Q"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_8_j_off():
    rank_1 = "8"
    suit_1 = "Diamonds"
    rank_2 = "J"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_8_10_off():
    rank_1 = "8"
    suit_1 = "Diamonds"
    rank_2 = "10"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_8_7_off():
    rank_1 = "8"
    suit_1 = "Diamonds"
    rank_2 = "7"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_6_7_off():
    rank_1 = "6"
    suit_1 = "Diamonds"
    rank_2 = "7"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_6_5_off():
    rank_1 = "6"
    suit_1 = "Diamonds"
    rank_2 = "5"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_4_5_off():
    rank_1 = "4"
    suit_1 = "Diamonds"
    rank_2 = "5"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_j_7_suited():
    rank_1 = "J"
    suit_1 = "Diamonds"
    rank_2 = "7"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_9_6_suited():
    rank_1 = "9"
    suit_1 = "Diamonds"
    rank_2 = "6"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_8_5_suited():
    rank_1 = "8"
    suit_1 = "Diamonds"
    rank_2 = "5"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_2_4_suited():
    rank_1 = "2"
    suit_1 = "Diamonds"
    rank_2 = "4"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_8_3_2_suited():
    rank_1 = "3"
    suit_1 = "Diamonds"
    rank_2 = "2"
    suit_2 = "Diamonds"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 8


def test_sklansky_7_k_low_suited():
    rank_1 = "4"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_q_8_suited():
    rank_1 = "Q"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_10_7_suited():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "7"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_6_5_suited():
    rank_1 = "6"
    suit_1 = "Clubs"
    rank_2 = "5"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_6_4_suited():
    rank_1 = "6"
    suit_1 = "Clubs"
    rank_2 = "5"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_5_3_suited():
    rank_1 = "3"
    suit_1 = "Clubs"
    rank_2 = "5"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_4_3_suited():
    rank_1 = "4"
    suit_1 = "Clubs"
    rank_2 = "3"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_4s():
    rank_1 = "4"
    suit_1 = "Clubs"
    rank_2 = "4"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_3s():
    rank_1 = "3"
    suit_1 = "Clubs"
    rank_2 = "3"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_2s():
    rank_1 = "2"
    suit_1 = "Clubs"
    rank_2 = "2"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_9_j_off():
    rank_1 = "9"
    suit_1 = "Clubs"
    rank_2 = "J"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_9_10_off():
    rank_1 = "9"
    suit_1 = "Clubs"
    rank_2 = "10"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_7_9_8_off():
    rank_1 = "9"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 7


def test_sklansky_6_10_a_off():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "A"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_10_k_off():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_10_q_off():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "Q"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_6s():
    rank_1 = "6"
    suit_1 = "Clubs"
    rank_2 = "6"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_5s():
    rank_1 = "6"
    suit_1 = "Clubs"
    rank_2 = "6"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_k_9_suited():
    rank_1 = "K"
    suit_1 = "Clubs"
    rank_2 = "9"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_j_8_suited():
    rank_1 = "J"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_6_8_suited():
    rank_1 = "6"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_6_4_5_suited():
    rank_1 = "4"
    suit_1 = "Clubs"
    rank_2 = "5"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 6


def test_sklansky_5_ace_low_suited():
    rank_1 = "A"
    suit_1 = "Clubs"
    rank_2 = "9"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_q_9_suited():
    rank_1 = "Q"
    suit_1 = "Clubs"
    rank_2 = "9"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_7_8_suited():
    rank_1 = "7"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_7_9_suited():
    rank_1 = "7"
    suit_1 = "Clubs"
    rank_2 = "9"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_7_6_suited():
    rank_1 = "7"
    suit_1 = "Clubs"
    rank_2 = "6"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_7s():
    rank_1 = "7"
    suit_1 = "Clubs"
    rank_2 = "7"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_j_k_off():
    rank_1 = "J"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_j_q_off():
    rank_1 = "J"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_5_j_10_off():
    rank_1 = "J"
    suit_1 = "Clubs"
    rank_2 = "10"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 5


def test_sklansky_4_a_j_off():
    rank_1 = "J"
    suit_1 = "Clubs"
    rank_2 = "A"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_q_k_off():
    rank_1 = "Q"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_8s():
    rank_1 = "8"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_10_k_suited():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_10_q_suited():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "Q"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_9_j_suited():
    rank_1 = "9"
    suit_1 = "Clubs"
    rank_2 = "J"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_9_10_suited():
    rank_1 = "9"
    suit_1 = "Clubs"
    rank_2 = "10"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_4_9_8_suited():
    rank_1 = "9"
    suit_1 = "Clubs"
    rank_2 = "8"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 4


def test_sklansky_3_a_10_suited():
    rank_1 = "A"
    suit_1 = "Hearts"
    rank_2 = "10"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_3_k_j_suited():
    rank_1 = "K"
    suit_1 = "Hearts"
    rank_2 = "J"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_3_q_j_suited():
    rank_1 = "Q"
    suit_1 = "Hearts"
    rank_2 = "J"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_3_q_a_off():
    rank_1 = "Q"
    suit_1 = "Hearts"
    rank_2 = "A"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_3_10_a_suited():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "A"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_3_10_j_suited():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "J"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_3_9s():
    rank_1 = "9"
    suit_1 = "Hearts"
    rank_2 = "9"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 3


def test_sklansky_2_ace_king_off():
    rank_1 = "K"
    suit_1 = "Hearts"
    rank_2 = "A"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 2


def test_sklansky_2_q_k_suited():
    rank_1 = "K"
    suit_1 = "Clubs"
    rank_2 = "Q"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 2


def test_sklansky_2_q_a_suited():
    rank_1 = "A"
    suit_1 = "Clubs"
    rank_2 = "Q"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 2


def test_sklansky_2_j_a_suited():
    rank_1 = "A"
    suit_1 = "Clubs"
    rank_2 = "J"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 2


def test_sklansky_2_tens():
    rank_1 = "10"
    suit_1 = "Clubs"
    rank_2 = "10"
    suit_2 = "Hearts"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 2


def test_sklansky_1_high_pair():
    rank_1 = "J"
    suit_1 = "Hearts"
    rank_2 = "J"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 1


def test_sklansky_1_ace_king_suited():
    rank_1 = "A"
    suit_1 = "Clubs"
    rank_2 = "K"
    suit_2 = "Clubs"
    card1 = Card().from_str(rank_1, suit_1)
    card2 = Card().from_str(rank_2, suit_2)

    val = sklansky_rank(card1, card2)
    assert val == 1



from texas_hold_em_utils.card import Card
from texas_hold_em_utils.relative_ranking import rank_hand


def test_rank_hand_best_possible_4_of_kind():
    hand = [Card().from_ints(0, 0), Card().from_ints(0, 1)]
    community_cards = [Card().from_ints(0, 2), Card().from_ints(0, 3), Card().from_ints(3, 3)]
    wins, losses, ties = rank_hand(hand, community_cards)
    assert losses == 0
    assert ties == 0
    assert wins == 2162 # just trusting the program here

def test_rank_hand_community_cards_best():
    hand = [Card().from_ints(0, 0), Card().from_ints(0, 1)]
    community_cards = [Card().from_ints(5, 2), Card().from_ints(5, 3), Card().from_ints(5, 1), Card().from_ints(7, 1), Card().from_ints(7, 0)]
    wins, losses, ties = rank_hand(hand, community_cards)
    assert losses == 318
    assert ties == 1662
    assert wins == 0



from texas_hold_em_utils.card import Card


def test_greater_than():
    card1 = Card().from_ints(0, 0)
    card2 = Card().from_ints(1, 0)
    assert card2.is_higher_than(card1)


def test_less_than():
    card1 = Card().from_ints(0, 0)
    card2 = Card().from_ints(1, 0)
    assert card1.is_lower_than(card2)


def test_equal():
    card1 = Card().from_ints(0, 0)
    card2 = Card().from_ints(0, 0)
    assert not card1.is_higher_than(card2)
    assert not card1.is_lower_than(card2)


def test_same_suit():
    card1 = Card().from_ints(0, 0)
    card2 = Card().from_ints(1, 0)
    assert card1.is_same_suit(card2)


def test_different_suit():
    card1 = Card().from_ints(0, 0)
    card2 = Card().from_ints(0, 1)
    assert not card1.is_same_suit(card2)




from texas_hold_em_utils.deck import Deck
from texas_hold_em_utils.hands import HandOfTwo, HandOfFive
from texas_hold_em_utils.player import Player


class Game:
    deck = None
    hands = []
    community_cards = []
    players = []
    dealer_position = 0
    big_blind = 0
    starting_chips = 0
    pot = 0
    all_day = 0
    round = 0
    player_ct = 0

    def __init__(self, num_players, big_blind=20, starting_chips=1000):
        """
        Initializes a game of Texas Hold 'Em with the given number of players, big blind, and starting chips
        :param num_players: the number of players in the game, by default these players are simple players
        :param big_blind: The big blind for the game
        :param starting_chips: The number of chips each player starts with
        """
        self.player_ct = num_players
        self.big_blind = big_blind
        self.starting_chips = starting_chips
        self.deck = Deck()
        self.deck.shuffle()
        for i in range(num_players):
            player = Player(i, starting_chips)
            player.hand_of_two = HandOfTwo([])
            self.players.append(player)

    def deal(self):
        """
        Deals two cards to each player in the game
        :return:
        """
        # two loops to simulate real dealing
        for player in self.players:
            player.hand_of_two.add_card(self.deck.draw())
        for player in self.players:
            player.hand_of_two.add_card(self.deck.draw())

    def flop(self):
        """
        Deals the flop and adds it to the community cards
        :return: an array of the community cards, in this case just the 3 flopped cards
        """
        # burn
        self.deck.draw()
        # turn
        self.community_cards = [self.deck.draw() for _ in range(3)]
        self.round += 1
        return self.community_cards

    def turn(self):
        """
        Deals the turn and adds it to the community cards
        :return: a list of the community cards, in this case  the 3 flop cards plus the turn card
        """
        # burn
        self.deck.draw()
        # turn
        self.community_cards.append(self.deck.draw())
        self.round += 1
        return self.community_cards

    def river(self):
        """
        Deals the river and adds it to the community cards
        :return: a list of the community cards, in this case  the 3 flop cards plus the turn card and the river card
        """
        # burn
        self.deck.draw()
        # turn
        self.community_cards.append(self.deck.draw())
        self.round += 1
        return self.community_cards

    def get_bets(self):
        """
        Gets the bets from each player in the round, keeping track of the pot and the required bet to stay in the round
        :return: the number of players still in the round
        """
        is_first_bet = True
        players_in_round = len([player for player in self.players if player.in_round])
        if self.round == 0:
            self.all_day = self.big_blind
            # small blind
            self.pot += self.players[(self.dealer_position + 1) % self.player_ct].bet(self.big_blind // 2)
            # big blind
            self.pot += self.players[(self.dealer_position + 2) % self.player_ct].bet(self.big_blind)
            last_bettor = self.dealer_position + 3 % self.player_ct  # player after big blind (hasn't really bet yet)
            next_bettor = last_bettor
        else:
            last_bettor = self.dealer_position + 1 % self.player_ct
            next_bettor = last_bettor
        while players_in_round > 1 and (is_first_bet or next_bettor % self.player_ct != last_bettor % self.player_ct):
            is_first_bet = False
            player = self.players[next_bettor % self.player_ct]
            if player.in_round:
                decision = player.decide(self.round, self.pot, self.all_day, self.big_blind, self.community_cards, players_in_round)
                if decision[0] == "raise":
                    self.all_day = player.round_bet
                    last_bettor = next_bettor
                elif decision[0] == "fold":
                    players_in_round -= 1
                self.pot += decision[1]
            next_bettor += 1
        return players_in_round

    def determine_round_winners(self):
        """
        Determines the winners of the round based on the best hand of five cards from each player
        :return: a list of players who have the best hand (usually just 1 player)
        """
        winners = []
        for player in self.players:
            if player.in_round:
                player.hand_of_five = HandOfFive(player.hand_of_two.cards, self.community_cards)
                if len(winners) == 0:
                    winners.append(player)
                elif player.hand_of_five > winners[0].hand_of_five:
                    winners = [player]
                elif player.hand_of_five == winners[0].hand_of_five:
                    winners.append(player)
        return winners

    def pay_out_winners(self):
        while self.pot > 0:
            winners = self.determine_round_winners()
            # find min bet of winners
            min_bet = min([player.round_bet for player in winners])
            payout = 0
            for player in self.players:
                contribution = min(player.round_bet, min_bet)
                player.round_bet -= contribution
                payout += contribution
                if player.round_bet == 0:
                    player.in_round = False
            indiv_payout = payout // len(winners)
            for winner in winners:
                winner.chips += indiv_payout
                self.pot -= indiv_payout
                payout -= indiv_payout
            i = 0
            while payout > 0:
                winners[i % len(winners)].chips += 1
                payout -= 1
                self.pot -= 1
                i += 1



from texas_hold_em_utils.hands import HandOfFive
from texas_hold_em_utils.deck import Deck


def rank_hand(hand, community_cards):
    player_hand = HandOfFive(hand, community_cards)
    deck1 = Deck()
    deck2 = Deck()
    wins = 0
    losses = 0
    ties = 0
    for card1 in deck1.cards:
        if card1 not in hand + community_cards:
            for card2 in deck2.cards:
                if card2 not in hand + community_cards and card2 != card1:
                    other_hand = HandOfFive([card1, card2], community_cards)
                    if player_hand > other_hand:
                        wins += 1
                    elif player_hand < other_hand:
                        losses += 1
                    else:
                        ties += 1
    return wins, losses, ties



"""
HAND_FUNCTIONS need to be operated on in the below order BUT ranks are the other way: Royal Flush = 7, and High Card = 0
"""
from texas_hold_em_utils.game_utils import *

HAND_FUNCTIONS = [
    find_royal_flush,
    find_straight_flush,
    find_four_of_a_kind,
    find_full_house,
    find_flush,
    find_straight,
    find_three_of_a_kind,
    find_two_pair,
    find_single_pair,
    find_high_card
]


class HandOfTwo:

    def __init__(self, cards):
        self.cards = cards

    def add_card(self, card):
        """
        Adds a card to the hand if there are less than 2 cards
        :param card:
        :return:
        """
        if len(self.cards) < 2:
            self.cards.append(card)
        else:
            raise ValueError("Hand already has 2 cards")


class HandOfFive:

    hand_cards = []
    community_cards = []
    hand_rank = None
    hand = []

    def __init__(self, hand_cards, community_cards):
        """

        :param hand_cards: list of 2 cards
        :param community_cards: list of 5 cards
        """
        self.hand_cards = hand_cards
        self.community_cards = community_cards
        self.determine_best(hand_cards, community_cards)

    def determine_best(self, hand_cards, community_cards):
        """
        Determines the best hand from the hand and community cards
        :param hand_cards: list of 2 cards
        :param community_cards: list of 5 cards
        :return: the 5 cards that make up the best hand, ordered so that the hand is easily compared to other hands
        Ex: a straight flush would be ordered from highest to lowest card, a full house would be ordered with the three
        of a kind first, then the pair
        """
        for i in range(len(HAND_FUNCTIONS)):
            self.hand = HAND_FUNCTIONS[i](hand_cards, community_cards)
            if self.hand is not None:
                self.hand_rank = 9 - i
                break

    def __gt__(self, other):
        """
        Compares two hands to see if the first hand is better than the second
        :param other: the other hand to compare to
        :return: True if this hand is better than the other, False otherwise (including when they are equal)
        """
        if self.hand_rank > other.hand_rank:
            return True
        elif self.hand_rank < other.hand_rank:
            return False
        for i in range(5):
            if self.hand[i].rank > other.hand[i].rank:
                return True
            elif self.hand[i].rank < other.hand[i].rank:
                return False
        return False

    def __eq__(self, other):
        """
        Compares two hands to see if they are equal
        :param other: the other hand to compare to
        :return: True if the hands are equal, False otherwise
        """
        if self.hand_rank == other.hand_rank:
            for i in range(5):
                if self.hand[i].rank != other.hand[i].rank:
                    return False
            return True
        return False

    def __lt__(self, other):
        """
        Compares two hands to see if the first hand is worse than the second
        :param other: the other hand to compare to
        :return: True if this hand is worse than the other, False otherwise (including when they are equal)
        """
        return not self.__gt__(other) and not self.__eq__(other)



class Card:
    ranks = ["2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"]
    suits = ["Hearts", "Diamonds", "Clubs", "Spades"]

    rank: int = 0
    suit: int = 0

    def __init__(self):
        self.rank = 0
        self.suit = 0

    def from_ints(self, rank: int, suit: int):
        """
        Set the rank and suit of the card from integers
        :param rank: 0 indexed ("2" = 0, "A" = 12) card rank
        :param suit: from 0-3: "Hearts", "Diamonds", "Clubs", "Spades"
        """
        self.rank = rank
        self.suit = suit
        return self

    def from_str(self, rank: str, suit: str):
        """
        Set the rank and suit of the card from strings
        :param rank: single character rank ["2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K", "A"]
        :param suit: suit ["Hearts", "Diamonds", "Clubs", "Spades"]
        :return:
        """
        self.rank = self.ranks.index(rank)
        self.suit = self.suits.index(suit)
        return self

    def name(self):
        return f"{self.ranks[self.rank]} of {self.suits[self.suit]}"

    def from_name(self, name: str):
        rank, suit = name.split(" of ")
        self.rank = self.ranks.index(rank)
        self.suit = self.suits.index(suit)
        return self

    def is_higher_than(self, card):
        """
        Returns True if this card is higher than the card passed in, False otherwise.
        :param card:
        :return:
        """
        return self.rank > card.rank

    def is_lower_than(self, card):
        """
        Returns True if this card is lower than the card passed in, False otherwise.
        :param card:
        :return:
        """
        return self.rank < card.rank

    def is_same_suit(self, card):
        return self.suit == card.suit

    def __gt__(self, other):
        return self.rank > other.rank

    def __eq__(self, other):
        return self.rank == other.rank and self.suit == other.suit

    def __str__(self):
        return self.name()
